

USE [AdventureWorks2012];
GO

SET STATISTICS TIME ON;
SET STATISTICS IO ON;
SET NOCOUNT ON;

/*
- crearsi un calendario di appoggio (una tabella calendar, tipica negli scenari di analisi) 
- recuperare le informazioni (in questo caso economiche) che vogliamo utilizzare per il calcolo 
- effettuare una join del resultset con se stesso per unoperazione di running total 
*/

WITH tYear
    AS ( SELECT 2005 AS year UNION ALL
         SELECT 2006 AS year UNION ALL
         SELECT 2007 AS year UNION ALL
         SELECT 2008 AS year UNION ALL
         SELECT 2009 AS year UNION ALL
         SELECT 2010 AS year ) , tMonth
    AS ( SELECT 'Q1' Quarter , 1 Month UNION ALL
         SELECT 'Q1' , 2 UNION ALL
         SELECT 'Q1' , 3 UNION ALL
         SELECT 'Q2' , 4 UNION ALL
         SELECT 'Q2' , 5 UNION ALL
         SELECT 'Q2' , 6 UNION ALL
         SELECT 'Q3' , 7 UNION ALL
         SELECT 'Q3' , 8 UNION ALL
         SELECT 'Q3' , 9 UNION ALL
         SELECT 'Q4' , 10 UNION ALL
         SELECT 'Q4' , 11 UNION ALL
         SELECT 'Q4' , 12 ) , tCalendar
    AS ( SELECT *
           FROM tYear CROSS JOIN tMonth ) , tSalesAmount
    AS ( SELECT SUM( subtotal )AS amount , 
                YEAR( orderdate )AS _year , 
                MONTH( orderdate )AS _month
           FROM sales.SalesOrderHeader tCalendar
           GROUP BY YEAR( orderdate ) , 
                    MONTH( orderdate )) , tCalendarSalesAmount
    AS ( SELECT s.Year , 
                s.Month , 
                s.Quarter , 
                tSalesAmount.amount
           FROM tCalendar s JOIN tSalesAmount ON tSalesAmount._year = s.Year AND tSalesAmount._month = s.month )
    SELECT [Year] , 
           Quarter , 
           [Month] , 
           Amount , 
           ( 
             SELECT SUM( b.Amount )
               FROM tCalendarSalesAmount AS b
               WHERE a.Year = b.year AND (b.[Month] - 1) / 3 = (a.[Month] - 1) / 3 AND b.[Month] <= a.[Month] )AS QTD , 
           ( 
             SELECT SUM( b.Amount )
               FROM tCalendarSalesAmount AS b
               WHERE a.Year = b.year AND b.[Month] <= a.[Month] )AS YTD
      FROM tCalendarSalesAmount AS a
      ORDER BY [Year] , Quarter , [Month];




SET STATISTICS TIME ON;
SET STATISTICS IO ON;
SET NOCOUNT ON; 
WITH t0
    AS ( SELECT SUM( subtotal )AS subtotal , 
                YEAR( orderdate )AS _year , 
                MONTH( orderdate )AS _month , 
                CASE MONTH( orderdate )
                WHEN 1 THEN 'Q1'
                WHEN 2 THEN 'Q1'
                WHEN 3 THEN 'Q1'
                WHEN 4 THEN 'Q2'
                WHEN 5 THEN 'Q2'
                WHEN 6 THEN 'Q2'
                WHEN 7 THEN 'Q3'
                WHEN 8 THEN 'Q3'
                WHEN 9 THEN 'Q3'
                WHEN 10 THEN 'Q4'
                WHEN 11 THEN 'Q4'
                WHEN 12 THEN 'Q4'
                END AS _quarter
           FROM sales.SalesOrderHeader
           GROUP BY YEAR( orderdate ) , 
                    MONTH( orderdate ))
    SELECT _year AS [Year] , 
           _quarter AS Quarter , 
           _month AS [Month] , 
           subtotal AS Amount , 
           SUM( subtotal ) OVER ( PARTITION BY _year , _quarter 
			 ORDER BY _year , _month ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW )AS QTD , 
           SUM( subtotal ) OVER ( PARTITION BY _year 
			 ORDER BY _year , _month ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW )AS YTD
      FROM t0
      ORDER BY _year , _month;
GO





/*
[SQL 2008] 
Table 'SalesOrderHeader'. Scan count 79, logical reads 76545

 SQL Server Execution Times:
   CPU time = 985 ms,  elapsed time = 329 ms.




[SQL 2012] 
Table 'SalesOrderHeader'. Scan count 1, logical reads 689

 SQL Server Execution Times:
   CPU time = 63 ms,  elapsed time = 66 ms.
*/


