

/* *** INIZIO CODICE TSQL *** */
USE CDays
GO  

IF OBJECT_ID('dbo.orders', 'U') IS NOT NULL 
    DROP TABLE dbo.orders
GO  

IF EXISTS ( SELECT  *
            FROM    INFORMATION_SCHEMA.ROUTINES
            WHERE   SPECIFIC_SCHEMA = N'dbo'
                    AND SPECIFIC_NAME = N'up_withoutCursor' ) 
    DROP PROCEDURE dbo.up_withoutCursor
GO  

IF EXISTS ( SELECT  *
            FROM    INFORMATION_SCHEMA.ROUTINES
            WHERE   SPECIFIC_SCHEMA = N'dbo'
                    AND SPECIFIC_NAME = N'up_withCursor' ) 
    DROP PROCEDURE dbo.up_withCursor
GO  

IF EXISTS ( SELECT  *
            FROM    INFORMATION_SCHEMA.ROUTINES
            WHERE   SPECIFIC_SCHEMA = N'dbo'
                    AND SPECIFIC_NAME = N'up_populateOrders' ) 
    DROP PROCEDURE dbo.up_populateOrders
GO  

/* Creo la tabella ordini su cui faremo le nostre analisi */
CREATE TABLE orders
    (
      idRecord INT PRIMARY KEY
                   IDENTITY(1, 1),
      qty SMALLINT,
      customerID CHAR(1)
    )
GO  

CREATE PROCEDURE dbo.up_populateOrders ( @numOrders int )
AS 
    SET NOCOUNT ON  

    TRUNCATE TABLE orders  

    DECLARE @ciclo INT
    SET @ciclo = 0  

    WHILE @ciclo < @numOrders
        BEGIN
            INSERT  orders
                    SELECT  CAST(RAND() * 100 AS INT),
                            'C'  

            SET @ciclo = @ciclo + 1
        END  

    SET @ciclo = 0
    WHILE @ciclo < @numOrders
        BEGIN
            INSERT  orders
                    SELECT  CAST(RAND() * 100 AS INT),
                            'A'  

            SET @ciclo = @ciclo + 1
        END  

    SET @ciclo = 0
    WHILE @ciclo < @numOrders
        BEGIN
            INSERT  orders
                    SELECT  CAST(RAND() * 100 AS INT),
                            'B'  

            SET @ciclo = @ciclo + 1
        END  

    SELECT  COUNT(*) AS numOrders
    FROM    orders  

    SET NOCOUNT OFF 
GO  

CREATE PROCEDURE dbo.up_withoutCursor
AS 
    SET NOCOUNT ON  

    DECLARE @StartDate datetime,
        @EndDate DATETIME
    SET @StartDate = getdate()  

    SELECT  customerID,
            O1.qty,
            O1.qty + ISNULL(( SELECT    SUM(O2.qty)
                              FROM      orders O2
                              WHERE     O2.idRecord < O1.idRecord
                                        AND O2.customerID = O1.customerID
                            ), 0) AS qtySum
    FROM    orders O1
    ORDER BY customerID,
            idRecord  

    SET @EndDate = getdate()
    SELECT  datediff(ms, @StartDate, @EndDate) AS 'Execution time in ms'
    SET NOCOUNT OFF
GO  

CREATE PROCEDURE dbo.up_withCursor
AS 
 

  SET NOCOUNT ON  

    DECLARE @StartDate datetime,
        @EndDate DATETIME
    SET @StartDate = getdate()  

    DECLARE @myQty INT
    DECLARE @customerID CHAR(1)
    DECLARE @CID CHAR(1)
    DECLARE @sum int
    DECLARE @tableResults TABLE
        (
          qty SMALLINT,
          qtySum INT,
          customerID CHAR(1)
        )  

    SET @myQty = 0
    SET @CID = ''  

    DECLARE myCursor CURSOR FAST_FORWARD READ_ONLY
        FOR SELECT  qty,
                    customerID
            FROM    orders
            ORDER BY customerID,
                    idRecord  

    OPEN myCursor  

    FETCH NEXT FROM myCursor INTO @myQty, @customerID  

    WHILE @@fetch_status = 0
        BEGIN
            IF @CID <> @customerID
            SET @sum = 0
            SET @CID = @customerID
            SET @sum = ( @sum + @myQty )
            INSERT  @tableResults
            VALUES  (
                      @myQty,
                      @sum,
                      @customerID
                    )  

            FETCH NEXT FROM myCursor INTO @myQty, @customerID
        END  

    SELECT  [customerID],
            [qty],
            [qtySum]
    FROM    @tableResults  

    CLOSE myCursor
    DEALLOCATE myCursor  

    SET @EndDate = getdate()
    SELECT  datediff(ms, @StartDate, @EndDate) AS 'Execution time in ms'
    SET NOCOUNT OFF 

GO 

/* *** FINE CODICE TSQL *** */




EXEC dbo.up_populateOrders 50
DBCC dropcleanbuffers
CHECKPOINT  
GO

EXEC dbo.up_withoutCursor
DBCC dropcleanbuffers

EXEC dbo.up_withCursor
DBCC dropcleanbuffers


/* SQLCLR */
DECLARE @StartDate datetime, @EndDate DATETIME
SET @StartDate = getdate()  
EXEC dbo.up_runningTotals
SET @EndDate = getdate()
SELECT  datediff(ms, @StartDate, @EndDate) AS 'Execution time in ms'


