USE Cdays
GO

IF OBJECT_ID('dbo.WaitStatLog') IS NOT NULL
  DROP TABLE dbo.WaitStatLog;
 
CREATE TABLE dbo.WaitStatLog
  (
    Id INT NOT NULL
           IDENTITY(1, 1)
           CONSTRAINT WaitStatLog_PK PRIMARY KEY CLUSTERED ,
    CaptDTime DATETIME2
      NOT NULL
      CONSTRAINT WaitStatLog_CaptDTime_Dflt DEFAULT SYSDATETIME()
  );
 
IF OBJECT_ID('dbo.WaitStatLogDtl') IS NOT NULL
  DROP TABLE dbo.WaitStatLogDtl;
 
CREATE TABLE dbo.WaitStatLogDtl
  (
    WaitStatLogId INT NOT NULL ,
    wait_type NVARCHAR(60) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
    waiting_tasks_count BIGINT NOT NULL ,
    wait_time_ms BIGINT NOT NULL ,
    max_wait_time_ms BIGINT NOT NULL ,
    signal_wait_time_ms BIGINT NOT NULL ,
    CONSTRAINT WaitStatLogDtl_PK PRIMARY KEY CLUSTERED
      ( WaitStatLogId, wait_type )
  );
 
IF OBJECT_ID('dbo.CaptureWaitStats') IS NOT NULL
  DROP PROCEDURE dbo.CaptureWaitStats;
GO
CREATE PROCEDURE dbo.CaptureWaitStats
AS
BEGIN 
  DECLARE @Id TABLE(Id INT);
  INSERT INTO dbo.WaitStatLog OUTPUT(INSERTED.Id) INTO @Id(Id) DEFAULT VALUES;
  INSERT INTO dbo.WaitStatLogDtl
  SELECT (SELECT Id FROM @Id), *
  FROM sys.dm_os_wait_stats;
END
GO


--> EXEC dbo.CaptureWaitStats; 
/* ******************* fine setup ********************************************** */


/*
LAG per calcolare i delta con la riga precedente
PARTITION BY per analizzare wait type su wait type
*/
WITH WaitDelta AS
(
SELECT  L.Id ,
        L.CaptDTime ,
        DATEDIFF(millisecond,LAG(L.CaptDTime, 1) OVER ( PARTITION BY D.wait_type ORDER BY L.Id ),L.CaptDTime) Interval,
        D.wait_type ,
        D.max_wait_time_ms ,
        D.waiting_tasks_count - LAG(D.waiting_tasks_count, 1) OVER ( PARTITION BY D.wait_type ORDER BY L.Id ) waiting_tasks_count,
        D.wait_time_ms - LAG(D.wait_time_ms, 1) OVER ( PARTITION BY D.wait_type ORDER BY L.Id ) wait_time_ms,
        D.signal_wait_time_ms - LAG(D.signal_wait_time_ms, 1) OVER ( PARTITION BY D.wait_type ORDER BY L.Id ) signal_wait_time_ms
FROM    dbo.WaitStatLog L
        JOIN dbo.WaitStatLogDtl D ON L.Id = D.WaitStatLogId
),
WaitTop AS
(
SELECT *,ROW_NUMBER() OVER(PARTITION BY Id ORDER BY wait_time_ms DESC) rn
FROM WaitDelta
WHERE waiting_tasks_count > 0
AND wait_type NOT IN (
        'CLR_SEMAPHORE', 'LAZYWRITER_SLEEP', 'RESOURCE_QUEUE', 'SLEEP_TASK',
        'SLEEP_SYSTEMTASK', 'SQLTRACE_BUFFER_FLUSH', 'WAITFOR', 'LOGMGR_QUEUE',
        'CHECKPOINT_QUEUE', 'REQUEST_FOR_DEADLOCK_SEARCH', 'XE_TIMER_EVENT', 'BROKER_TO_FLUSH',
        'BROKER_TASK_STOP', 'CLR_MANUAL_EVENT', 'CLR_AUTO_EVENT', 'DISPATCHER_QUEUE_SEMAPHORE',
        'FT_IFTS_SCHEDULER_IDLE_WAIT', 'XE_DISPATCHER_WAIT', 'XE_DISPATCHER_JOIN', 'BROKER_EVENTHANDLER',
        'TRACEWRITE', 'FT_IFTSHC_MUTEX', 'SQLTRACE_INCREMENTAL_FLUSH_SLEEP','SLEEP_BPOOL_FLUSH',
        'CXPACKET','DBMIRROR_EVENTS_QUEUE','DBMIRRORING_CMD')
)
SELECT *
FROM WaitTop
WHERE rn<6
ORDER BY Id DESC;

